diff -uNr trunk/common/csp.c trunk-new/common/csp.c
--- trunk/common/csp.c	Tue Jul 17 08:11:18 2007
+++ trunk-new/common/csp.c	Mon Dec 24 19:55:17 2007
@@ -64,7 +64,7 @@
         int     i;
         for( i = 0; i < w; i++ )
         {
-            *d++ = ( s[0] + s[1] + s[i_src] + s[i_src+1] + 1 ) >> 2;
+            *d++ = ( s[0] + s[1] + s[i_src] + s[i_src+1] + 2 ) >> 2;
             s += 2;
         }
         dst += i_dst;
@@ -254,24 +254,64 @@
     }
 }
 
-/* Same value than in XviD */
-#define BITS 8
-#define FIX(f) ((int)((f) * (1 << BITS) + 0.5))
-
-#define Y_R   FIX(0.257)
-#define Y_G   FIX(0.504)
-#define Y_B   FIX(0.098)
-#define Y_ADD 16
-
-#define U_R   FIX(0.148)
-#define U_G   FIX(0.291)
-#define U_B   FIX(0.439)
-#define U_ADD 128
-
-#define V_R   FIX(0.439)
-#define V_G   FIX(0.368)
-#define V_B   FIX(0.071)
-#define V_ADD 128
+#ifdef BT_709
+//BT.709
+#define Kb 0.0722
+#define Kr 0.2126
+#else
+//BT.601
+#define Kb 0.114
+#define Kr 0.299
+#endif
+
+#define Kg (1.0 - Kb - Kr)
+#define Sb (1.0 - Kb)
+#define Sr (1.0 - Kr)
+
+#ifdef PC_SCALE
+//PC Scale
+//0 <= Y <= 255
+//0 <= U <= 255
+//0 <= V <= 255
+#define Ky 1.0
+#define Ku (0.5 / Sb)
+#define Kv (0.5 / Sr)
+#define Ay 0.0
+#define Au 127.5
+#define Av 127.5
+#else
+//TV Scale
+//16 <= Y <= 235
+//16 <= U <= 240
+//16 <= V <= 240
+#define Ky (1.0        * 219.0 / 255.0)
+#define Ku ((0.5 / Sb) * 224.0 / 255.0)
+#define Kv ((0.5 / Sr) * 224.0 / 255.0)
+#define Ay 16.0
+#define Au 128.0
+#define Av 128.0
+#endif
+
+#define BITS      22
+#define INT_FIX   (1 << BITS)
+#define INT_ROUND (INT_FIX >> 1)
+#define FIX(f)    ((uint32_t)((f) * INT_FIX + 0.5))
+
+#define Y_R   FIX(Kr*Ky)
+#define Y_G   FIX(Kg*Ky)
+#define Y_B   FIX(Kb*Ky)
+#define Y_ADD ((uint32_t)(Ay * INT_FIX + INT_ROUND + 0.5))
+
+#define U_R   FIX(Kr*Ku)
+#define U_G   FIX(Kg*Ku)
+#define U_B   FIX(Sb*Ku)
+#define U_ADD ((uint32_t)((Au * INT_FIX + INT_ROUND) * 4 + 0.5))
+
+#define V_R   FIX(Sr*Kv)
+#define V_G   FIX(Kg*Kv)
+#define V_B   FIX(Kb*Kv)
+#define V_ADD ((uint32_t)((Av * INT_FIX + INT_ROUND) * 4 + 0.5))
+
 #define RGB_TO_I420( name, POS_R, POS_G, POS_B, S_RGB ) \
 static void name( x264_mc_functions_t *mc,              \
                   x264_frame_t *frm, x264_image_t *img, \
@@ -300,20 +340,20 @@
                                                         \
         for( w = i_width; w > 0; w -= 2 )               \
         {                                               \
-            int cr = 0,cg = 0,cb = 0;                   \
-            int r, g, b;                                \
+            uint32_t cr = 0,cg = 0,cb = 0;              \
+            uint32_t r, g, b;                           \
                                                         \
             /* Luma */                                  \
             cr = r = ss[POS_R];                         \
             cg = g = ss[POS_G];                         \
             cb = b = ss[POS_B];                         \
                                                         \
-            yy[0] = Y_ADD + ((Y_R * r + Y_G * g + Y_B * b) >> BITS);    \
+            yy[0] = (Y_ADD + Y_R * r + Y_G * g + Y_B * b) >> BITS;    \
                                                         \
             cr+= r = ss[POS_R+i_src];                   \
             cg+= g = ss[POS_G+i_src];                   \
             cb+= b = ss[POS_B+i_src];                   \
-            yy[i_y] = Y_ADD + ((Y_R * r + Y_G * g + Y_B * b) >> BITS);  \
+            yy[i_y] = (Y_ADD + Y_R * r + Y_G * g + Y_B * b) >> BITS;  \
             yy++;                                       \
             ss += S_RGB;                                \
                                                         \
@@ -321,21 +361,21 @@
             cg+= g = ss[POS_G];                         \
             cb+= b = ss[POS_B];                         \
                                                         \
-            yy[0] = Y_ADD + ((Y_R * r + Y_G * g + Y_B * b) >> BITS);    \
+            yy[0] = (Y_ADD + Y_R * r + Y_G * g + Y_B * b) >> BITS;    \
                                                         \
             cr+= r = ss[POS_R+i_src];                   \
             cg+= g = ss[POS_G+i_src];                   \
             cb+= b = ss[POS_B+i_src];                   \
-            yy[i_y] = Y_ADD + ((Y_R * r + Y_G * g + Y_B * b) >> BITS);  \
+            yy[i_y] = (Y_ADD + Y_R * r + Y_G * g + Y_B * b) >> BITS;  \
             yy++;                                       \
             ss += S_RGB;                                \
                                                         \
             /* Chroma */                                \
-            *uu++ = (uint8_t)(U_ADD + ((-U_R * cr - U_G * cg + U_B * cb) >> (BITS+2)) ); \
-            *vv++ = (uint8_t)(V_ADD + (( V_R * cr - V_G * cg - V_B * cb) >> (BITS+2)) ); \
+            *uu++ = (uint8_t)((U_ADD + U_B * cb - U_R * cr - U_G * cg) >> (BITS+2)); \
+            *vv++ = (uint8_t)((V_ADD + V_R * cr - V_G * cg - V_B * cb) >> (BITS+2)); \
         }                                               \
                                                         \
-        src += 2*i_src;                                   \
+        src += 2*i_src;                                 \
         y += 2*frm->i_stride[0];                        \
         u += frm->i_stride[1];                          \
         v += frm->i_stride[2];                          \